package de.tudarmstadt.informatik.hostage.dao;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.tudarmstadt.informatik.hostage.R;
import de.tudarmstadt.informatik.hostage.model.Profile;
import de.tudarmstadt.informatik.hostage.ui2.adapter.ProfileManagerListAdapter;
import de.tudarmstadt.informatik.hostage.ui2.model.ProfileListItem;

/**
 * @author Alexander Brakowski
 * @created 10.02.14 20:24
 */
public class ProfileManager {
	private static ProfileManager INSTANCE = null;

	private Map<Integer, Profile> mProfiles;
	private ProfileManagerListAdapter mProfileListAdapter = null;

	private int mProfileId = 4;
	private Profile mCurrentActivatedProfile = null;

	public static ProfileManager getInstance(){
		if(INSTANCE == null){
			INSTANCE = new ProfileManager();
		}

		return INSTANCE;
	}

	private ProfileManager(){
		this.mProfiles = new HashMap<Integer, Profile>();
		this.mProfiles.put(0, new Profile(
				0,
				"Windows Vista",
				"Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat.",
				R.drawable.ic_profile_vista,
				false
		));

		this.mProfiles.put(1, new Profile(
				1,
				"Windows 7",
				"Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat.",
				R.drawable.ic_profile_w7,
				false
		));

		this.mProfiles.put(2, new Profile(
				2,
				"Unix Distro",
				"Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat.",
				R.drawable.ic_profile_unix,
				false
		));

		this.mProfiles.put(3, new Profile(
				3,
				"Random",
				"Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat.",
				R.drawable.ic_service_green,
				false
		));

		this.mProfiles.put(4, new Profile(
				4,
				"Mix",
				"Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat.",
				R.drawable.ic_service_green,
				false
		));
	}

	public List<Profile> getProfilesList(){
		return new ArrayList<Profile>(getProfilesCollection());
	}

	public Collection<Profile> getProfilesCollection(){
		return mProfiles.values();
	}

	public Map<Integer, Profile> getMapProfiles(){
		return mProfiles;
	}

	public Profile getProfile(int id){
		if(this.mProfiles.containsKey(id)){
			return this.mProfiles.get(id);
		}

		return null;
	}

	public void addProfile(Profile profile){
		if(profile.id == -1){
			profile.id = mProfileId++;
		}

		this.mProfiles.put(profile.id, profile);
		if(this.mProfileListAdapter != null){
			this.mProfileListAdapter.add(profile);
		}
	}

	public void deleteProfile(Profile profile){
		if(this.mProfiles.containsKey(profile.id)){
			this.mProfiles.remove(profile.id);

			if(this.mProfileListAdapter != null){
				this.mProfileListAdapter.remove(profile);
			}
		}
	}

	public void activeProfile(Profile profile){
		if(profile.equals(this.mCurrentActivatedProfile)) return;

		if(this.mCurrentActivatedProfile != null){
			this.mCurrentActivatedProfile.activated = false;
		}

		profile.activated = true;
		this.mCurrentActivatedProfile = profile;
	}

	public void setProfileListAdapter(ProfileManagerListAdapter profileListAdapter){
		this.mProfileListAdapter = profileListAdapter;
	}


	public ProfileManagerListAdapter getProfileListAdapter(){
		return this.mProfileListAdapter;
	}
}
